#include "testlib.h"
#include <iostream>
#include <fstream>
#include <vector>
#include <algorithm>

using namespace std;

int n, m;
vector<string> input;

string IntToStr(int value){
	string result = "";
	if (value == 0) return 0;
	while (value > 0) {
		result += ('0' + value % 10);
		value /= 10;
	}
	reverse(result.begin(), result.end());
	return result;
}

bool valid(int i, int j) {
	return 0 <= i && i < n + 2 && 0 <= j && j < m + 2;
}

int dx[] = { 0, 0, -1, 1 };
int dy[] = { -1, 1, 0, 0 };

const int nmax = 3e3;

int C[nmax][nmax];

void dfs(const vector<string>& t, int i, int j, int c) {
	if (!valid(i, j)) return;
	if (C[i][j] != 0) return;
	if (t[i][j] != '.') return;
	C[i][j] = c;
	for (int k = 0; k < 4; k ++) {
		int i1 = i + dx[k];
		int j1 = j + dy[k];
		dfs(t, i1, j1, c);
	}
}

string mul(const string& s, int val) {
	string result = "";
	for (int i = 0; i < val; i ++)
		result += s;
	return result;
}

string pattern = "[CT.]";


int readAns(InStream in, bool isJury) {
	int result = 0;
	vector<string> s;
	s.push_back(mul(".",(m + 2)));
	for (int i = 0; i < n; i ++) {
		string line = in.readToken(pattern + "{" + IntToStr(m) + "," + IntToStr(m) + "}") ;
		for (int j = 0; j < m; j ++) {
			if (input[i][j] == '.') {
				if (line[j] != '.') {
					quitf(isJury ? _fail : _wa, "Impossible to put something to position (%d, %d)", i + 1, j + 1);
				}
				continue;
			}
			if (input[i][j] == 'T') {
				if (line[j] != 'T') {
					quitf(isJury ? _fail : _wa, "Impossible to remove table from position (%d, %d)", i + 1, j + 1);
				}
				continue;
			}
			if (input[i][j] == 'C') {
				if (line[j] == 'T') {
					quitf(isJury ? _fail : _wa, "Impossible to put table to position (%d, %d)", i + 1, j + 1);
				}
				if (line[j] == '.') {
					result += 1;
				}
				continue;
			}
		}
		s.push_back("." + line + ".");
	}
	s.push_back(mul(".", (m + 2)));
	for (int i = 0; i < nmax; i ++)
		for (int j = 0; j < nmax; j ++)
			C[i][j] = 0;	
	int clrs = 0;
	for (int i = 0; i < n + 2; i ++)
		for (int j = 0; j < m + 2; j ++)
			if (s[i][j] == '.' && C[i][j] == 0)
				dfs(s, i, j, ++clrs);
	if (clrs != 1) 
		quitf(isJury ? _fail : _wa, "Area is not connected");
	return result;
}



int main(int argc, char* argv[]) {
	registerTestlibCmd(argc, argv);
	n = inf.readInt(); n *= 3;
	m = inf.readInt(); m *= 3;
	for (int i = 0; i < n; i ++)
		input.push_back(inf.readToken(pattern + "{" + IntToStr(m) + "," + IntToStr(m) + "}"));	
	int ja = readAns(ans, true);
	int ca = readAns(ouf, false);
	if (ja < ca)
		quitf(_wa, "Answer is not optimal");
	if (ja > ca)
		quitf(_fail, "Contestant found better answer");
	quitf(_ok, "Ok, %d chairs removed", ja); 
	return 0;
}
