#include <cassert>
#include <cstring>
#include <cstdio>
#include <cstdlib>
#include <vector>
#include <algorithm>
#include <cmath>
#include <signal.h>

using namespace std;

FILE *flog;

const int MAX_JUMP = 100;
const int MAX_MOVES = 80;

struct vt
{
    int x, y;
    vt(){}

    vt(int _x, int _y)
    {
        x = _x, y = _y;
    }
    friend vt operator +(vt a, vt b)
    {
        return vt(a.x + b.x, a.y + b.y);
    }
    friend vt operator -(vt a, vt b)
    {
        return vt(a.x - b.x, a.y - b.y);
    }
    friend bool operator ==(vt a, vt b)
    {
        return make_pair(a.x, a.y) == make_pair(b.x, b.y);
    }
};

const char* name[4] = {
    "NOT GOING TO USE",
    "1st rook",
    "2nd rook",
    "king",
};

int exitInt(int code)
{
    fclose(flog);
    fclose(stdout);
    char c;
    while (scanf("%c", &c) != -1) {}
    exit(code);
    return 0;
}

bool check(vt f, vt f1, vt f2) {
    if (f1.x == f2.x) {
        if (f.x == f1.x) {
            if (f.y >= f1.y && f.y <= f2.y) {
                //fprintf(stderr, "%d %d %d %d %d %d = 1\n", f.x, f.y, f1.x, f1.y, f2.x, f2.y);
                return false;
            }
            if (f.y >= f2.y && f.y <= f1.y) {
                //fprintf(stderr, "%d %d %d %d %d %d = 2\n", f.x, f.y, f1.x, f1.y, f2.x, f2.y);
                return false;
            }
        }
    } else {
        if (f.y == f1.y) {
            if (f.x >= f1.x && f.x <= f2.x) {
                //fprintf(stderr, "%d %d %d %d %d %d = 3\n", f.x, f.y, f1.x, f1.y, f2.x, f2.y);
                return false;
            }
            if (f.x >= f2.x && f.x <= f1.x) {
                //fprintf(stderr, "%d %d %d %d %d %d = 4\n", f.x, f.y, f1.x, f1.y, f2.x, f2.y);
                return false;
            }
        }
    }
    return true;
}

vt dv[] =
{
    vt(-1, 1) , vt(0, 1),  vt(1, 1),
    vt(-1, 0) ,            vt(1, 0),
    vt(-1, -1), vt(0, -1), vt(1, -1)
};

bool good(vt K, vt *R)
{
    for (int i = 0; i < 2; i++)
        if ((R[i].x == K.x) ^ (R[i].y == K.y))
            return false;
    for (int i = 0; i < 8; i++)
        if (R[2] == K + dv[i])
            return false;
    return true;
}

vt try_to_eat(vt K, vt* R)
{
    for (int i = 0; i < 3; i++)
        for (int v = 0; v < 8; v++)
            if (K + dv[v] == R[i] && good(K + dv[v], R))
            {
                assert(i != 3);
                return dv[v];
            }
    return vt(0, 0);
}

double greed(vt K, vt* R)
{
    double ans = 0;
    for (int i = 0; i < 2; i++)
        ans += -1.0 / ((double)((K.x - R[i].x) * (K.x - R[i].x)) + ((double)(K.y - R[i].y) * (K.y - R[i].y)));
    return ans;
}

const int P = 424243;
double random(vt K, vt *R)
{
    return (((((((K.x * P + K.y) * P + R[0].x) * P + R[0].y) * P +
R[1].x) * P + R[1].y) * P + R[2].x) * P + R[2].y);
}

double coward(vt K, vt* R)
{
    double ans = 0;
    for (int i = 0; i < 3; i++)
        ans -= (K.x - R[i].x) * (K.x - R[i].x) + (K.y - R[i].y) * (K.y - R[i].y);
    return ans;
}

double coward_king(vt K, vt* R)
{
    double ans = 0;
    for (int i = 2; i < 3; i++)
        ans -= (K.x - R[i].x) * (K.x - R[i].x) + (K.y - R[i].y) * (K.y - R[i].y);
    return ans;
}

double greed_king(vt K, vt* R)
{
    double ans = 0;
    for (int i = 2; i < 3; i++)
        ans += (K.x - R[i].x) * (K.x - R[i].x) + (K.y - R[i].y) * (K.y - R[i].y);
    return ans;
}

typedef double (*rate_function)(vt K, vt *R);

rate_function RF[] = {random, greed, coward, coward_king, greed_king};

const double EPS = 1e-8;

// K: king
// R: array of rooks
// type:
// * 0 for greed
// * 1 for random
// * 2 for coward
// * 3 for coward (out from king)
// * 4 for greed (to king)
vt move_king(vt K, vt* R, int type)
{
    vt v = try_to_eat(K, R);
    if (v.x != 0 || v.y != 0)
        return v;
    vector<vt> poss;
    rate_function rf = RF[type];
    for (int i = 0; i < 8; i++)
    {
        vt k = K + dv[i];
        if (good(k, R))
        {
            poss.push_back(k);
            double rrb = rf(poss.back(), R);
            double rrf = rf(poss.front(), R);
            //fprintf(flog, "%lf %lf\n", rrb, rrf); <---- вот если это раскомментировать, то начинает работать
            if (rrb > rrf + EPS)
                poss.pop_back();
            else if (rrb < rrf - EPS)
                poss.erase(poss.begin(), poss.end() - 1);
        }
    }
    if (poss.empty())
    {
        fprintf(flog, "It's nowhere for king to go :-(\n");
        return vt(0, 0);
    }
    vt tmp = poss[rand() % poss.size()];
    tmp.x -= K.x;
    tmp.y -= K.y;
    return tmp;
}

#define log(...) do { fprintf(flog, __VA_ARGS__); fflush(flog); } while(0)
#define die(r, ...) do { fprintf(flog, __VA_ARGS__); fflush(flog); exitInt(r); } while (0)

vt fg[4];
const int FW = 10;
const int FH = 50;
void write_field()
{
#ifndef DEBUG
    return;
#endif
    for (int i = max(max(fg[0].y, fg[1].y), max(fg[2].y, fg[3].y)); i >= min(min(fg[0].y, fg[1].y), min(fg[2].y, fg[3].y)); i--)
    {
        for (int j = min(min(fg[0].x, fg[1].x), min(fg[2].x, fg[3].x)); j <= max(max(fg[0].x, fg[1].x), max(fg[2].x, fg[3].x)); j++)
        {
            if (fg[0] == vt(j, i))
                log("K");
            else
            {
                bool was = false;
                for (int t = 1; t <= 2; t++)
                    if (fg[t] == vt(j, i))
                    {
                        log("%d", (t == 3) ? 0 : t);
                        was = true;
                    }
                if (fg[3] == vt(j, i))
                {
                    log("X");
                    was = true;
                }
                if (!was)
                    log(".");
            }
        }
        log("\n");
    }
}

int main(int argc, char* argv[])
{
    signal(SIGPIPE, SIG_IGN);
    srand(424243);
    if (argc < 3) {
        fprintf(stderr, "wrong parameters");
        exit(99);
    }

    FILE *f = fopen(argv[1], "r");
    flog = fopen(argv[2], "w");


    int mode;

    for (int i = 0; i < 4; ++i) {
        if (fscanf(f, "%d%d", &fg[i].x, &fg[i].y) != 2)
            die(99, "wrong test");
    }

    if (fscanf(f, "%d", &mode) != 1)
        die(99, "wrong test");
    if (mode < 0 || mode >= 5)
        die(99, "wrong mode");
    for (int i = 0; i < 4; i++)
        for (int j = 0; j < i; j++)
            if (fg[i] == fg[j])
                die(99, "wrong test, figures coincide\n");

    for (int i = 1; i <= 2; i++)
        if (fg[i].x == fg[0].x || fg[i].y == fg[0].y)
            die(99, "wrong test, rook %d eats king\n", i);

    for (int i = 0; i < 4; ++i)
        printf("%d %d ", fg[i].x, fg[i].y);
    printf("\n");
    fflush(stdout);

    log("Starting position: black king on (%d %d), white rooks on (%d %d) (%d %d), white king on (%d %d)\n", fg[0].x, fg[0].y, fg[1].x, fg[1].y, fg[2].x, fg[2].y, fg[3].x, fg[3].y);

    int count = 0;
    while (count < MAX_MOVES) {
        char* fig = NULL;
        int dx, dy;
        const char* strs[] = {"\0", "R1", "R2", "K"};
        write_field();

        if (scanf("%as %d %d", &fig, &dx, &dy) != 3)
            die(4, "Error in reading Max's move\n");
        if (strlen(fig) >= 4)
            die(4, "Wrong figure");

        int num = -1;
        for (int i = 0; i < 4; i++)
            if (strcmp(strs[i], fig) == 0)
                num = i;
        if (num <= 0)
            die(4, "Wrong figure\n");

        log("\nTurn %d\n", count + 1);
        log("Max moves %s to (%d %d)...\n", name[num], fg[num].x + dx, fg[num].y + dy);

        if (fig[0] == 'K')
        {
            if (abs(dx) > 1 || abs(dy) > 1 || (dx == 0 && dy == 0))
                die(4, "It's not a valid king move");

            for (int i = 0; i < 4; i++)
                if (fg[i] == fg[3] + vt(dx, dy))
                    die(4, "There is a figure on the way of king");
            for (int ddx = -1; ddx <= 1; ddx++)
                for (int ddy = -1; ddy <= 1; ddy++)
                    if (fg[0] + vt(ddx, ddy) == fg[3] + vt(dx, dy))
                        die(4, "Cannot attack black king with a white king");
        }
        else
        {
            if ((dx * dy) != 0)
                die(4, "It's not a valid rook move\n");

            if (abs(dx) > MAX_JUMP || abs(dy) > MAX_JUMP)
                die(4, "It's too long for a move\n");

            if (abs(dx) + abs(dy) == 0)
                die(4, "Cannot move a rook to the same place it was before\n");

            for (int i = 0; i < 4; i++) {
                if (i != num) {
                    if (!check(fg[i], fg[num], fg[num] + vt(dx, dy))) {
                        die(5, "There is a figure on the way.\n");
                    }
                }
            }
        }


        fg[num].x += dx;
        fg[num].y += dy;

        write_field();

        log("OK.\n");

        vt res = move_king(fg[0], fg + 1, mode);

        if (res.x == 0 && res.y == 0) {
            for (int i = 1; i < 3; i++) {
                if (fg[i].x == fg[0].x || fg[i].y == fg[0].y ) {
                    printf("0 0");
                    fflush(stdout);
                    die(0, "Game over! Checkmate - Max wins!\n");
                }
            }
            log("Game over! Statemate - Alex wins!\n");
            exitInt(5);
        }

        for (int i = 1; i < 3; ++i)
            if (fg[i].x == fg[0].x + res.x && fg[i].y == fg[0].y + res.y)
                die(5, "Alex moves king to (%d %d) and eates %s!\nGame over! Max can't win any more!\n", res.x + fg[0].x, res.y + fg[0].y, name[i]);
            else if (fg[i].x == fg[0].x + res.x || fg[i].y == fg[0].y + res.y)
                die(99, "wrong king move\n");


        fg[0].x += res.x;
        fg[0].y += res.y;

        log("Alex moved king to (%d %d)\n", fg[0].x, fg[0].y);
        log("# Black king on (%d %d), white rooks on (%d %d) (%d %d), white king on (%d %d)\n", fg[0].x, fg[0].y, fg[1].x, fg[1].y, fg[2].x, fg[2].y, fg[3].x, fg[3].y);
        printf("%d %d\n", res.x, res.y);
        fflush(stdout);

        count++;
    }
    die(5, "Game over! Too many moves - Alex wins!");
    return 0;
}
