#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

enum
{
  RUN_OK               = 0,
  RUN_COMPILE_ERR      = 1,
  RUN_RUN_TIME_ERR     = 2,
  RUN_TIME_LIMIT_ERR   = 3,
  RUN_PRESENTATION_ERR = 4,
  RUN_WRONG_ANSWER_ERR = 5,
  RUN_CHECK_FAILED     = 6,
  RUN_PARTIAL          = 7,
  RUN_ACCEPTED         = 8,
  RUN_IGNORED          = 9,
  RUN_DISQUALIFIED     = 10,
  RUN_PENDING          = 11,
  RUN_MEM_LIMIT_ERR    = 12,
  RUN_SECURITY_ERR     = 13,
  RUN_STYLE_ERR        = 14,
  RUN_WALL_TIME_LIMIT_ERR = 15,
  RUN_PENDING_REVIEW   = 16,
  RUN_REJECTED         = 17,
};

static void die(const char *format, ...)
    __attribute__((noreturn, format(printf, 1, 2)));
static void die(const char *format, ...)
{
    va_list args;
    char buf[1024];

    va_start(args, format);
    vsnprintf(buf, sizeof(buf), format, args);
    va_end(args);

    fprintf(stderr, "valuer fatal error: %s\n", buf);
    exit(RUN_CHECK_FAILED);
}

// experimental interactive valuer
int main(int argc, char *argv[])
{
    if (!getenv("EJUDGE")) die("expected EJUDGE environment");
    if (!getenv("EJUDGE_INTERACTIVE")) die("expected EJUDGE_INTERACTIVE");
    if (!getenv("EJUDGE_USER_SCORE")) die("expected EJUDGE_USER_SCORE");
    FILE *fcmt = fopen(argv[1], "w");

    int total_count = -2;
    if (scanf("%d", &total_count) != 1 || total_count != -1)
        die("expected total_count == -1");

    // test groups:
    // 1-2:   from statement, 0
    // 3-20:  30
    // 21-50: 70 offline

    enum { G0_FIRST = 1, G0_LAST = 2, G0_SCORE = 0,
           G0_COUNT = G0_LAST - G0_FIRST + 1 };
    enum { G1_FIRST = 3, G1_LAST = 20, G1_SCORE = 30,
           G1_COUNT = G1_LAST - G1_FIRST + 1 };
    enum { G2_FIRST = 21, G2_LAST = 50, G2_SCORE = 70,
           G2_COUNT = G2_LAST - G2_FIRST + 1 };

    int group0_tests_passed = 0;
    int group1_tests_passed = 0;
    int group2_tests_passed = 0;

    int cur_test = 0, status, test_score, ttime;
    while (scanf("%d%d%d", &status, &test_score, &ttime) == 3) {
        ++cur_test;
        if (cur_test >= G0_FIRST && cur_test <= G0_LAST) {
            if (status == RUN_OK) ++group0_tests_passed;
            printf("-1\n");
            fflush(stdout);
        } else if (cur_test >= G1_FIRST && cur_test <= G1_LAST) {
            if (status == RUN_OK) {
                ++group1_tests_passed;
                printf("-1\n");
            } else {
//                printf("-%d\n", G2_FIRST);
                if (cur_test < G1_LAST) {
                    fprintf(fcmt, "Тестирование на тестах %d-%d не выполнялось, "
                            "так как тест %d не пройден, и оценка за группу тестов %d - 0 баллов.\n",
                            cur_test + 1, G1_LAST, cur_test, 1);
                }
//                cur_test = G1_LAST;
                break;
            }
            fflush(stdout);
        } else if (cur_test >= G2_FIRST && cur_test <= G2_LAST) {
            if (status == RUN_OK) {
                ++group2_tests_passed;
                printf("-1\n");
            } else {
                printf("-%d\n", G2_LAST + 1);
                cur_test = G2_LAST;
            }
            fflush(stdout);
        } else {
            die("unexpected test number");
        }
    }

    int total_score = 0;
    int user_status = RUN_PARTIAL;
    int user_score = 0;
    int user_tests_passed = 0;

    if (group1_tests_passed == G1_COUNT) {
        total_score += G1_SCORE;
        user_score += G1_SCORE;
    }
    if (group2_tests_passed == G2_COUNT) {
        total_score += G2_SCORE;
    }
    if (user_score == G1_SCORE && group0_tests_passed == G0_COUNT) user_status = RUN_OK;
    user_tests_passed = group0_tests_passed + group1_tests_passed;
    printf("%d %d %d %d\n",
           total_score, user_status, user_score, user_tests_passed);
    if (user_score != G1_SCORE) {
        fprintf(fcmt, "Решение не будет тестироваться на тестах %d-%d после окончания тура!\n",
                G2_FIRST, G2_LAST);
    }
    fclose(fcmt);
    return 0;
}

/*
 * Local variables:
 *  c-basic-offset: 4
 * End:
 */
