#include <cstdio>
#include <cassert>
#include "testlib.h"
using namespace std;

struct vt
{
    int x, y;
    vt(int _x, int _y)
    {
        x = _x, y = _y;
    }
    vt(){}
    friend vt operator +(vt a, vt b)
    {
        return vt(a.x + b.x, a.y + b.y);
    }
    friend vt operator -(vt a, vt b)
    {   
        return vt(a.x - b.x, a.y - b.y);
    }
    friend int operator *(vt a, vt b)
    {
        return a.x * b.x + a.y * b.y;
    }
    friend int operator ^(vt a, vt b)
    {
        return a.x * b.y - b.x * a.y;
    }
    friend bool operator ||(vt a, vt b)
    {
        return (a ^ b) == 0;
    }
};

int sign(int x)
{
    return (x < 0) ? -1 : x > 0;
}

bool in(vt x, vt a, vt b)
{
    return (((a - x) || (b - x)) && ((x - a) * (x - b) <= 0));
}

bool inter(vt a, vt b, vt c, vt d)
{
    if ((b - a) || (c - d))
    {
        if (!((b - a) || (c - a)))
            return false;
        return (in(a, c, d) || in(b, c, d) || in(c, a, b) || in(d, a, b));
    }
    else
        return sign((b - a) ^ (c - a)) * sign((b - a) ^ (d - a)) <= 0 && 
               sign((d - c) ^ (b - c)) * sign((d - c) ^ (a - c)) <= 0;
}

const int N = 300;
vt P[N];
int MAX = 30000;

int n;

inline int mymod(int x)
{
    return (x >= n) ? x - n : x;
}

bool find(const vector<pair<int, int> >& V, pair<int, int> pr)
{
    vector<pair<int, int> >::const_iterator it = lower_bound(V.begin(), V.end(), pr);
    return it != V.end() && *it == pr;
}

int main(int argc, char* argv[])
{
    registerTestlibCmd(argc, argv);
    n = inf.readInt(3, N);
    inf.readEoln();
    for (int i = 0; i < n; i++)
    {
        P[i].x = inf.readInt(-MAX, MAX);
        inf.readSpace();
        P[i].y = inf.readInt(-MAX, MAX);
        inf.readEoln();
    }
    
    inf.readEof();
    vector<pair<int, int> > A, B;
    int a = ouf.readInt(), b = ans.readInt();
    for (int i = 0; i < a; i++)
    {
        int u = ouf.readInt(1, n);
        ouf.readSpace();
        int v = ouf.readInt(1, n);
        ouf.readEoln();
        A.push_back(make_pair(u - 1, v - 1));
    }
    for (int i = 0; i < b; i++)
    {
        int u = ans.readInt(1, n);
        ans.readSpace();
        int v = ans.readInt(1, n);
        ans.readEoln();
        B.push_back(make_pair(u - 1, v - 1));
    }
    int x = ouf.readInt(0, -1 + (1 << 30));
    int y = ans.readInt(0, -1 + (1 << 30));
    ouf.readEoln();
    ouf.readEof();
    ans.readEoln();
    ans.readEof();
    vector<pair<int, int> > _A = A, _B = B;
    for (int i = 0; i < A.size(); i++)
        if (A[i].first >= A[i].second)
            quitf(_pe, "wrong points order in diagonal\n");
    sort(A.begin(), A.end());
    if (A != _A)
        quitf(_pe, "diagonals aren't sorted\n");
    A.resize(unique(A.begin(), A.end()) - A.begin());
    if (A != _A)
        quitf(_pe, "diagonals aren't uniqued\n");
    for (int i = 0; i < B.size(); i++)
        if (B[i].first >= B[i].second)
            quitf(_fail, "our wrong points order in diagonal\n");
    sort(B.begin(), B.end());
    if (B != _B)
        quitf(_fail, "our diagonals aren't sorted\n");
    B.resize(unique(B.begin(), B.end()) - B.begin());
    if (B != _B)
        quitf(_fail, "our diagonals aren't uniqued\n");
    for (int i = 0; i < A.size(); i++)
        if (!find(B, A[i]))
            quitf(_wa, "diagonal %d %d isn't inner\n", 1 + A[i].first, 1 + A[i].second);
    for (int i = 0; i < B.size(); i++)
        if (!find(A, B[i]))
            quitf(_wa, "diagonal %d %d is inner, but he didn't found it\n", 1 + B[i].first, 1 + B[i].second);
    assert(A == B);
    if (x != y)
        quitf(_wa, "correct answer is %d, not %d\n", y, x);
        
    return 0;
}
